@@ -1,6 +1,5 @@
 //! Read and write onboard flash memory.
-//! The Flash memory is organized as 72-bit wide memory cells (64 bits plus 8 ECC bits) that
-//! can be used for storing both code and data constants.
+//! Note that on dual bank variants, only bank 1 is supported for now.
 
 // Note that most of the code for L5 and U5 is feature-gated due to different
 // register names, differentiating secure and non-secure. We keep them in the same file
@@ -16,6 +15,10 @@
 // Based on `stm32l4xx-hal`.
 
 use crate::pac::FLASH;
+
+#[cfg(feature = "h7")]
+use crate::pac::flash::BANK;
+
 use core;
 
 use cfg_if::cfg_if;
@@ -39,10 +42,9 @@ enum _DualBank {
 }
 
 #[derive(Clone, Copy)]
-pub enum BanksToErase {
-    Bank1,
-    Bank2,
-    Both,
+pub enum Bank {
+    B1,
+    B2,
 }
 
 #[derive(Copy, Clone, Debug)]
@@ -60,12 +62,16 @@ pub enum Error {
     Failure,
 }
 
-#[cfg(not(feature = "l5"))]
+// todo: Bank 2 support on H7 and others.
+
+#[cfg(not(any(feature = "l5", feature = "h7")))]
 /// Check and clear all non-secure error programming flags due to a previous
 /// programming. If not, PGSERR is set.
-fn check_illegal(flash: &FLASH) -> Result<(), Error> {
+fn check_illegal(regs: &FLASH) -> Result<(), Error> {
     // todo: QC this fn and its l5 variant.
-    let sr = flash.sr.read();
+
+    let sr = regs.sr.read();
+
     cfg_if! {
         if #[cfg(any(feature = "f3"))] {
             if sr.pgerr().bit_is_set() || sr.wrprterr().bit_is_set() {
@@ -84,13 +90,26 @@ fn check_illegal(flash: &FLASH) -> Result<(), Error> {
     Ok(())
 }
 
+#[cfg(feature = "h7")]
+/// Check and clear all non-secure error programming flags due to a previous
+/// programming. If not, PGSERR is set.
+fn check_illegal(regs: &BANK) -> Result<(), Error> {
+    // todo: QC this fn and its l5 variant.
+    let sr = regs.sr.read();
+    if sr.pgserr().bit_is_set() || sr.wrperr().bit_is_set() {
+        return Err(Error::Illegal);
+    }
+
+    Ok(())
+}
+
 #[cfg(feature = "l5")]
 /// Check and clear all non-secure error programming flags due to a previous
 /// programming. If not, NSPGSERR is set.
-fn check_illegal(flash: &FLASH, security: Security) -> Result<(), Error> {
+fn check_illegal(regs: &FLASH, security: Security) -> Result<(), Error> {
     match security {
         Security::NonSecure => {
-            let sr = flash.nssr.read();
+            let sr = regs.nssr.read();
             if sr.nspgaerr().bit_is_set()
                 || sr.nspgserr().bit_is_set()
                 || sr.nsprogerr().bit_is_set()
@@ -100,7 +119,7 @@ fn check_illegal(flash: &FLASH, security: Security) -> Result<(), Error> {
             }
         }
         Security::Secure => {
-            let sr = flash.secsr.read();
+            let sr = regs.secsr.read();
             if sr.secpgaerr().bit_is_set()
                 || sr.secpgaerr().bit_is_set()
                 || sr.secprogerr().bit_is_set()
@@ -155,10 +174,15 @@ impl Flash {
     #[cfg(not(feature = "l5"))]
     /// Unlock the flash memory, allowing writes. See L4 Reference manual, section 3.3.5.
     pub fn unlock(&mut self) -> Result<(), Error> {
-        self.regs.keyr.write(|w| unsafe { w.bits(FLASH_KEY1) });
-        self.regs.keyr.write(|w| unsafe { w.bits(FLASH_KEY2) });
+        #[cfg(not(feature = "h7"))]
+        let regs = &self.regs;
+        #[cfg(feature = "h7")]
+        let regs = self.regs.bank1();
 
-        if self.regs.cr.read().lock().bit_is_clear() {
+        regs.keyr.write(|w| unsafe { w.bits(FLASH_KEY1) });
+        regs.keyr.write(|w| unsafe { w.bits(FLASH_KEY2) });
+
+        if regs.cr.read().lock().bit_is_clear() {
             Ok(())
         } else {
             Err(Error::Failure)
@@ -167,7 +191,10 @@ impl Flash {
 
     #[cfg(not(feature = "l5"))]
     pub fn lock(&mut self) {
+        #[cfg(not(feature = "h7"))]
         self.regs.cr.modify(|_, w| w.lock().set_bit());
+        #[cfg(feature = "h7")]
+        self.regs.bank1().cr.modify(|_, w| w.lock().set_bit());
     }
 
     #[cfg(feature = "l5")]
@@ -188,9 +215,14 @@ impl Flash {
     pub fn erase_page(&mut self, page: usize) -> Result<(), Error> {
         self.unlock()?;
 
+        #[cfg(not(feature = "h7"))]
+        let regs = &self.regs;
+        #[cfg(feature = "h7")]
+        let regs = &self.regs.bank1(); // todo: Don't hard code bank1
+
         // 1. Check that no Flash memory operation is ongoing by checking the BSY bit in the Flash
         // status register (FLASH_SR).
-        let sr = self.regs.sr.read();
+        let sr = regs.sr.read();
         if sr.bsy().bit_is_set() {
             self.lock();
             return Err(Error::Busy);
@@ -198,7 +230,7 @@ impl Flash {
 
         // 2. Check and clear all error programming flags due to a previous programming. If not,
         // PGSERR is set.
-        if check_illegal(&self.regs).is_err() {
+        if check_illegal(regs).is_err() {
             self.lock();
             return Err(Error::Illegal);
         };
@@ -213,22 +245,22 @@ impl Flash {
             if #[cfg(feature = "f3")] {
                 // F3 RM: "Erase procedure"
                 // Set the PER bit in the FLASH_CR register
-                self.regs.cr.modify(|_, w| w.per().set_bit());
+                regs.cr.modify(|_, w| w.per().set_bit());
 
                 // Program the FLASH_CR register
-                // self.regs.ar.modify(|_, w| w.far().bits(page as u8));
-                self.regs.ar.write(|w| unsafe { w.bits(page as u32) }); // todo: Is this right?
-            } else if #[cfg(feature = "f4")] {
+                // regs.ar.modify(|_, w| w.far().bits(page as u8));
+                regs.ar.write(|w| unsafe { w.bits(page as u32) }); // todo: Is this right?
+            } else if #[cfg(any(feature = "f4", feature = "h7"))] {
                 // Set the SER bit and select the sector out of the 12 sectors (for STM32F405xx/07xx and
                 // STM32F415xx/17xx) and out of 24 (for STM32F42xxx and STM32F43xxx) in the main
                 // memory block you wish to erase (SNB) in the FLASH_CR register
-                self.regs.cr.modify(|_, w| unsafe {
+                regs.cr.modify(|_, w| unsafe {
                     w.ser().set_bit();
                     w.snb().bits(page as u8) // todo: Probably not right?
                 });
 
             } else if #[cfg(any(feature = "g0", feature = "g4", feature = "wb", feature = "wl"))] {
-                 self.regs.cr.modify(|_, w| unsafe {
+                 regs.cr.modify(|_, w| unsafe {
                     w.pnb()
                     .bits(page as u8)
                     .per()
@@ -237,12 +269,12 @@ impl Flash {
             } else {
                 match page {
                     0..=255 => {
-                        self.regs.cr.modify(|_, w| unsafe {
+                        regs.cr.modify(|_, w| unsafe {
                             w.bker().clear_bit().pnb().bits(page as u8).per().set_bit()
                         });
                     }
                     256..=511 => {
-                        self.regs.cr.modify(|_, w| unsafe {
+                        regs.cr.modify(|_, w| unsafe {
                             w.bker()
                                 .set_bit()
                                 .pnb()
@@ -261,28 +293,30 @@ impl Flash {
         // 4. Set the STRT bit in the FLASH_CR register.
         cfg_if! {
             if #[cfg(any(feature = "f3", feature = "f4", feature = "wb", feature = "wl"))] {
-                self.regs.cr.modify(|_, w| w.strt().set_bit());
+                regs.cr.modify(|_, w| w.strt().set_bit());
             } else {
                 #[cfg(any(feature = "g0", feature = "g4"))]
-                self.regs.cr.modify(|_, w| w.strt().set_bit());
+                regs.cr.modify(|_, w| w.strt().set_bit());
                 #[cfg(not(any(feature = "g0", feature = "g4")))]
-                self.regs.cr.modify(|_, w| w.start().set_bit());
+                regs.cr.modify(|_, w| w.start().set_bit());
             }
         }
 
         // 5. Wait for the BSY bit to be cleared in the FLASH_SR register.
-        while self.regs.sr.read().bsy().bit_is_set() {}
+        while regs.sr.read().bsy().bit_is_set() {}
 
         // todo on F3: "Read the erased option bytes and verify" as final step
         cfg_if! {
-            if #[cfg(any(feature = "f3", feature = "f4"))] {
+            if #[cfg(any(feature = "f3", feature = "f4", feature = "h7"))] {
                 // Check the EOP flag in the FLASH_SR register (it is set when the erase operation has
                 // succeeded), and then clear it by software. (todo)
 
+                // todo: Check H7 procedure here.
+
                 // Clear the EOP flag
-                self.regs.sr.modify(|_, w| w.eop().set_bit());
+                regs.sr.modify(|_, w| w.eop().set_bit());
             } else {
-                self.regs.cr.modify(|_, w| w.per().clear_bit());
+                regs.cr.modify(|_, w| w.per().clear_bit());
             }
         }
 
@@ -406,17 +440,29 @@ impl Flash {
     }
 
     #[cfg(not(feature = "l5"))]
-    pub fn erase_bank(&mut self, banks: BanksToErase) -> Result<(), Error> {
+    /// Erase one or both banks.
+    pub fn erase_bank(&mut self, bank: Bank) -> Result<(), Error> {
         // todo: DRY
+        // (H7): 2. Unlock the FLASH_CR1/2 register, as described in Section 4.5.1: FLASH configuration
+        // protection (only if register is not already unlocked).
         self.unlock()?;
 
+        #[cfg(not(feature = "h7"))]
+        let regs = &self.regs;
+        #[cfg(feature = "h7")]
+        let regs = &match bank {
+            Bank::B1 => self.regs.bank1(),
+            Bank::B2 => self.regs.bank2(),
+        };
+
         // To perform a bank Mass Erase, follow the procedure below:
         // RM0351 Rev 7 105/1903
         // RM0351 Embedded Flash memory (FLASH)
-        // 139
         // 1. Check that no Flash memory operation is ongoing by checking the BSY bit in the
         // FLASH_SR register.
-        let sr = self.regs.sr.read();
+        // (H7): 1. Check and clear (optional) all the error flags due to previous programming/erase
+        // operation. Refer to Section 4.7: FLASH error management for details.
+        let sr = regs.sr.read();
         if sr.bsy().bit_is_set() {
             self.lock();
             return Err(Error::Busy);
@@ -424,7 +470,7 @@ impl Flash {
 
         // 2. Check and clear all error programming flags due to a previous programming. If not,
         // PGSERR is set.
-        if check_illegal(&self.regs).is_err() {
+        if check_illegal(regs).is_err() {
             self.lock();
             return Err(Error::Illegal);
         };
@@ -433,39 +479,40 @@ impl Flash {
         // (FLASH_CR). Both banks can be selected in the same operation.
 
         cfg_if! {
-            if #[cfg(any(feature = "f3", feature = "f4", feature = "wb", feature = "wl"))] {
-                self.regs.cr.modify(|_, w| w.mer().clear_bit());
+            if #[cfg(any(feature = "f3", feature = "f4", feature = "wb", feature = "wl", feature = "h7"))] {
+                #[cfg(not(feature = "h7"))]
+                regs.cr.modify(|_, w| w.mer().clear_bit());
+                #[cfg(feature = "h7")]
+                // 3. Set the BER1/2 bit in the FLASH_CR1/2 register corresponding to the targeted bank.
+                regs.cr.modify(|_, w| w.ber().clear_bit());
 
                 // 4. Set the STRT bit in the FLASH_CR register.
-                self.regs.cr.modify(|_, w| w.strt().set_bit());
+                #[cfg(not(feature = "h7"))]
+                regs.cr.modify(|_, w| w.strt().set_bit());
+                #[cfg(feature = "h7")]
+                // Set the START1/2 bit in the FLASH_CR1/2 register to start the bank erase operation.
+                // Then wait until the QW1/2 bit is cleared in the corresponding FLASH_SR1/2 register.
+                regs.cr.modify(|_, w| w.start().set_bit());
             } else if #[cfg(any(feature = "g0"))] {
-                self.regs.cr.modify(|_, w| w.mer().clear_bit());
+                regs.cr.modify(|_, w| w.mer().clear_bit());
             } else if #[cfg(any(feature = "g4"))] {
-                self.regs.cr.modify(|_, w| w.mer1().clear_bit());
+                regs.cr.modify(|_, w| w.mer1().clear_bit());
             } else {
-                match banks {
-                    BanksToErase::Bank1 => {
-                        self.regs.cr.modify(|_, w| w.mer1().clear_bit());
-                    }
-                    BanksToErase::Bank2 => {
-                        self.regs.cr.modify(|_, w| w.mer2().clear_bit());
-                    }
-                    BanksToErase::Both => {
-                        self.regs.cr.modify(|_, w| w.mer1().clear_bit());
-                        self.regs.cr.modify(|_, w| w.mer2().clear_bit());
-                    }
+                match bank {
+                    Bank::B1 => regs.cr.modify(|_, w| w.mer1().clear_bit()),
+                    Bank::B2 => regs.cr.modify(|_, w| w.mer2().clear_bit()),
                 }
 
                 // 4. Set the STRT bit in the FLASH_CR register.
                 #[cfg(any(feature = "g4", feature = "wl"))]
-                self.regs.cr.modify(|_, w| w.strt().set_bit());
+                regs.cr.modify(|_, w| w.strt().set_bit());
                 #[cfg(not(feature = "g4"))]
-                self.regs.cr.modify(|_, w| w.start().set_bit());
+                regs.cr.modify(|_, w| w.start().set_bit());
             }
         }
 
         // 5. Wait for the BSY bit to be cleared in the FLASH_SR register.
-        while self.regs.sr.read().bsy().bit_is_set() {}
+        while regs.sr.read().bsy().bit_is_set() {}
 
         self.lock();
 
@@ -474,7 +521,7 @@ impl Flash {
 
     #[cfg(feature = "l5")]
     /// Mass erase: L5 RM section 6.3.6
-    pub fn erase_bank(&mut self, banks: BanksToErase, security: Security) -> Result<(), Error> {
+    pub fn erase_bank(&mut self, bank: Bank, security: Security) -> Result<(), Error> {
         self.unlock(security)?;
 
         match security {
@@ -498,17 +545,9 @@ impl Flash {
 
                 // 3. Set the MER1 bit or/and MER2 (depending on the bank) in the Flash control register
                 // (FLASH_CR). Both banks can be selected in the same operation.
-                match banks {
-                    BanksToErase::Bank1 => {
-                        self.regs.nscr.modify(|_, w| w.nsmer1().clear_bit());
-                    }
-                    BanksToErase::Bank2 => {
-                        self.regs.nscr.modify(|_, w| w.nsmer2().clear_bit());
-                    }
-                    BanksToErase::Both => {
-                        self.regs.nscr.modify(|_, w| w.nsmer1().clear_bit());
-                        self.regs.nscr.modify(|_, w| w.nsmer2().clear_bit());
-                    }
+                match bank {
+                    Bank::B1 => self.regs.nscr.modify(|_, w| w.nsmer1().clear_bit()),
+                    Bank::B2 => self.regs.nscr.modify(|_, w| w.nsmer2().clear_bit()),
                 }
 
                 // 4. Set the NSSTRT bit in the FLASH_NSCR register.
@@ -529,17 +568,9 @@ impl Flash {
                     return Err(Error::Illegal);
                 };
 
-                match banks {
-                    BanksToErase::Bank1 => {
-                        self.regs.seccr.modify(|_, w| w.secmer1().clear_bit());
-                    }
-                    BanksToErase::Bank2 => {
-                        self.regs.seccr.modify(|_, w| w.secmer2().clear_bit());
-                    }
-                    BanksToErase::Both => {
-                        self.regs.seccr.modify(|_, w| w.secmer1().clear_bit());
-                        self.regs.seccr.modify(|_, w| w.secmer2().clear_bit());
-                    }
+                match bank {
+                    Bank::B1 => self.regs.seccr.modify(|_, w| w.secmer1().clear_bit()),
+                    Bank::B2 => self.regs.seccr.modify(|_, w| w.secmer2().clear_bit()),
                 }
 
                 self.regs.seccr.modify(|_, w| w.secstrt().set_bit());
@@ -553,6 +584,8 @@ impl Flash {
         Ok(())
     }
 
+    // todo: For H7 etc, accept a bank argument.
+
     #[cfg(not(feature = "l5"))]
     /// Write the contents of a page. Must be erased first. See L4 RM, section 3.3.7.
     pub fn write_page(&mut self, page: usize, data: &[u64]) -> Result<(), Error> {
@@ -563,7 +596,12 @@ impl Flash {
         // Flash status register (FLASH_SR).
         self.unlock()?;
 
-        let sr = self.regs.sr.read();
+        #[cfg(not(feature = "h7"))]
+        let regs = &self.regs;
+        #[cfg(feature = "h7")]
+        let regs = &self.regs.bank1();
+
+        let sr = regs.sr.read();
         if sr.bsy().bit_is_set() {
             self.lock();
             return Err(Error::Busy);
@@ -571,17 +609,21 @@ impl Flash {
 
         // 2. Check and clear all error programming flags due to a previous programming. If not,
         // PGSERR is set.
-        if check_illegal(&self.regs).is_err() {
+        if check_illegal(regs).is_err() {
             self.lock();
             return Err(Error::Illegal);
         };
 
         // 3. Set the PG bit in the Flash control register (FLASH_CR).
-        self.regs.cr.modify(|_, w| w.pg().set_bit());
+        regs.cr.modify(|_, w| w.pg().set_bit());
 
         // 4. Perform the data write operation at the desired memory address, inside main memory
         // block or OTP area. Only double word can be programmed.
+
+        #[cfg(not(feature = "h7"))]
         let mut address = page_to_address(page) as *mut u32;
+        #[cfg(feature = "h7")]
+        let mut address = sector_to_address(page, Bank::B1) as *mut u32;
 
         for dword in data {
             unsafe {
@@ -594,18 +636,18 @@ impl Flash {
             }
 
             // 5. Wait until the BSY bit is cleared in the FLASH_SR register.
-            while self.regs.sr.read().bsy().bit_is_set() {}
+            while regs.sr.read().bsy().bit_is_set() {}
 
             // 6. Check that EOP flag is set in the FLASH_SR register (meaning that the programming
             // operation has succeed), and clear it by software.
-            if self.regs.sr.read().eop().bit_is_set() {
-                self.regs.sr.modify(|_, w| w.eop().set_bit()); // Clear
+            if regs.sr.read().eop().bit_is_set() {
+                regs.sr.modify(|_, w| w.eop().set_bit()); // Clear
             }
         }
 
         // 7. Clear the PG bit in the FLASH_CR register if there no more programming request
         // anymore.
-        self.regs.cr.modify(|_, w| w.pg().clear_bit());
+        regs.cr.modify(|_, w| w.pg().clear_bit());
 
         self.lock();
 
@@ -717,24 +759,74 @@ impl Flash {
         Ok(())
     }
 
+    #[cfg(not(feature = "h7"))]
     /// Read a single 64-bit memory cell, indexed by its page, and an offset from the page.
     pub fn read(&self, page: usize, offset: isize) -> u64 {
         let addr = page_to_address(page) as *const u64;
         unsafe { core::ptr::read(addr.offset(offset)) }
     }
 
+    // #[cfg(not(feature = "h7"))]
     /// Read flash memory at a given page and offset into a buffer.
     pub fn read_to_buffer(&self, page: usize, offset: isize, buff: &mut [u8]) {
+        // H742 RM, section 4.3.8:
+        // Single read sequence
+        // The recommended simple read sequence is the following:
+        // 1. Freely perform read accesses to any AXI-mapped area.
+        // 2. The embedded Flash memory effectively executes the read operation from the read
+        // command queue buffer as soon as the non-volatile memory is ready and the previously
+        // requested operations on this specific bank have been served.
+
         // todo: This is untested.
+        #[cfg(not(feature = "h7"))]
         let addr = page_to_address(page) as *const u8; // todo is this right?
+                                                       // let addr = page_to_address(page).as_ptr(); // todo is this right?
+        #[cfg(feature = "h7")]
+        // todo: Don't hard-code bank1.
+        let addr = sector_to_address(page, Bank::B1) as *const u8; // todo is this right?
+                                                                   // let addr = sector_to_address(page, Bank::B1).as_ptr(); // todo is this right?
 
         for val in buff {
             *val = unsafe { core::ptr::read(addr.offset(offset)) }
         }
     }
+
+    // #[cfg(feature = "h7")]
+    // /// Read flash memory at a given page and offset into a buffer.
+    // ///    H742 RM, section 4.3.8:
+    // /// Single read sequence
+    // /// The recommended simple read sequence is the following:
+    // /// 1. Freely perform read accesses to any AXI-mapped area.
+    // /// 2. The embedded Flash memory effectively executes the read operation from the read
+    // /// command queue buffer as soon as the non-volatile memory is ready and the previously
+    // /// requested operations on this specific bank have been served.
+    // pub fn read_to_buffer(&self, sector: usize, offset: isize, buff: &mut [u8]) {
+    //     //  todo: H7 can read in 64 bit, 32 bit, 16 bit, or one byte granularities.
+    //     //     // todo: H7 uses 256 bit words for writing and erasing.
+    //     // for val in buff {
+    //     for i in buff.len() {
+    //         let val = buff[i]
+    //         *val = unsafe { core::ptr::read(addr.offset(i)) };
+    //     }
+    // }
 }
 
-/// Calculate the address of the start of a given page. Each page is 2,048 Kb.
+#[cfg(not(feature = "h7"))]
+/// Calculate the address of the start of a given page. Each page is 2,048 Kb for non-H7.
+/// For H7, sectors are 128Kb, with 8 sectors per bank.
 fn page_to_address(page: usize) -> usize {
-    0x0800_0000 + page as usize * 2048
+    0x0800_0000 + page * 2048
+}
+
+#[cfg(feature = "h7")]
+/// Calculate the address of the start of a given page. Each page is 2,048 Kb for non-H7.
+/// For H7, sectors are 128Kb, with 8 sectors per bank.
+fn sector_to_address(sector: usize, bank: Bank) -> usize {
+    let starting_pt = match bank {
+        Bank::B1 => 0x0800_0000,
+        // todo: This isn't the same bank2 starting point for all H7 variants!
+        Bank::B2 => 0x0810_0000,
+    };
+
+    starting_pt + sector * 0x2_0000
 }
