@@ -398,7 +398,9 @@ macro_rules! set_exti_g0 {
 
 /// Represents a single GPIO pin. Allows configuration, and reading/setting state.
 pub struct Pin {
+    /// The GPIO Port letter. Eg A, B, C.
     pub port: Port,
+    /// The pin number: 1 - 15.
     pub pin: u8,
 }
 
@@ -452,7 +454,7 @@ impl Pin {
     }
 
     /// Create a new pin, with a specific mode. Enables the RCC peripheral clock to the port,
-    /// if not already enabled.
+    /// if not already enabled. Example: `let pa1 = Pin::new(Port::A, 1);`
     pub fn new(port: Port, pin: u8, mode: PinMode) -> Self {
         assert!(pin <= 15, "Pin must be 0 - 15.");
 
@@ -493,7 +495,7 @@ impl Pin {
                     cfg_if! {
                         if #[cfg(feature = "f3")] {
                             if rcc.ahbenr.read().iopben().bit_is_clear() {
-                                rcc_en_reset!(ahb1, iopa, rcc);
+                                rcc_en_reset!(ahb1, iopb, rcc);
                             }
                         } else if #[cfg(feature = "h7")] {
                             if rcc.ahb4enr.read().gpioben().bit_is_clear() {
@@ -523,7 +525,7 @@ impl Pin {
                     cfg_if! {
                         if #[cfg(feature = "f3")] {
                             if rcc.ahbenr.read().iopcen().bit_is_clear() {
-                                rcc_en_reset!(ahb1, iopa, rcc);
+                                rcc_en_reset!(ahb1, iopc, rcc);
                             }
                         } else if #[cfg(feature = "h7")] {
                             if rcc.ahb4enr.read().gpiocen().bit_is_clear() {
@@ -553,7 +555,7 @@ impl Pin {
                     cfg_if! {
                         if #[cfg(feature = "f3")] {
                             if rcc.ahbenr.read().iopden().bit_is_clear() {
-                                rcc_en_reset!(ahb1, iopa, rcc);
+                                rcc_en_reset!(ahb1, iopd, rcc);
                             }
                         } else if #[cfg(feature = "h7")] {
                             if rcc.ahb4enr.read().gpioden().bit_is_clear() {
@@ -590,7 +592,7 @@ impl Pin {
                     cfg_if! {
                         if #[cfg(feature = "f3")] {
                             if rcc.ahbenr.read().iopeen().bit_is_clear() {
-                                rcc_en_reset!(ahb1, iopa, rcc);
+                                rcc_en_reset!(ahb1, iope, rcc);
                             }
                         } else if #[cfg(feature = "h7")] {
                             if rcc.ahb4enr.read().gpioeen().bit_is_clear() {
@@ -630,7 +632,7 @@ impl Pin {
                     cfg_if! {
                         if #[cfg(feature = "f3")] {
                             if rcc.ahbenr.read().iopfen().bit_is_clear() {
-                                rcc_en_reset!(ahb1, iopa, rcc);
+                                rcc_en_reset!(ahb1, iopf, rcc);
                             }
                         } else if #[cfg(feature = "h7")] {
                             if rcc.ahb4enr.read().gpiofen().bit_is_clear() {
@@ -670,7 +672,7 @@ impl Pin {
                     cfg_if! {
                         if #[cfg(feature = "f3")] {
                             if rcc.ahbenr.read().iophen().bit_is_clear() {
-                                rcc_en_reset!(ahb1, iopa, rcc);
+                                rcc_en_reset!(ahb1, ioph, rcc);
                             }
                         } else if #[cfg(feature = "h7")] {
                             if rcc.ahb4enr.read().gpiohen().bit_is_clear() {
